﻿Imports System.Globalization
Imports System.Runtime.InteropServices
Imports System.Text
Module GlobalesModul
    Public Programmverzeichnis As String
    Public BrailleUebersetzungstabelle As String = ""
    Public BrailleAusgabetabelle As String = ""
    Public Eintrag() As String
    Public Brailletabellenliste As New List(Of String)
    Public Dateiinhalt As System.IO.StreamReader
    Public AktuelleZeile As String = ""
End Module

Class LibLouis
    Public Shared Function Hyphenate(input As String) As String
        Dim refLength As Int32 = input.Length + 32 ' size heuristic 
        Dim buffer As New StringBuilder(refLength + 1)
        NativeMethods.lou_hyphenate("Braille\Brailletabellen\hyph_de_DE.dic", input, input.Length, buffer, 0)
        buffer.Length = refLength + 1
        Return buffer.ToString()
    End Function

    Public Shared Function Translate(input As String) As String
        Dim refLength As Integer = input.Length + 32 ' size heuristic 
        Dim buffer As New StringBuilder(refLength)
        Dim typeform() = Enumerable.Repeat(NativeMethods.Typeforms.plain_text, refLength).ToArray()
        NativeMethods.lou_translateString(Programmverzeichnis & "\Braille\Brailletabellen\" & BrailleUebersetzungstabelle, input, input.Length, buffer, refLength, typeform, Nothing, 0)
        buffer.Length = refLength
        Return buffer.ToString
    End Function

    Public Shared Function DotsToChar(input() As Short) As String
        Dim buffer As New StringBuilder(input.Length)
        NativeMethods.lou_dotsToChar(Programmverzeichnis & "\Braille\Brailletabellen\" & BrailleAusgabetabelle, input, buffer, input.Length, NativeMethods.TranslationModes.ucBrl)
        buffer.Length = input.Length
        Return buffer.ToString()
    End Function

    Public Shared Function CharToDots(input As String) As Short()
        Dim buffer(input.Length - 1) As Short
        NativeMethods.lou_charToDots(Programmverzeichnis & "\Braille\Brailletabellen\" & BrailleAusgabetabelle, input, buffer, input.Length, NativeMethods.TranslationModes.ucBrl)
        Return buffer
    End Function

    Public Shared Function GetDotNumbers(input() As Short) As String
        Dim result As New StringBuilder
        Dim l As Int32
        For Each s In input
            s = s And &HFFS
            l = 1
            While s <> 0
                If s Mod 2 = 1 Then result.Append(l)
                s \= 2S
                l += 1
            End While
            result.Append("-")
        Next
        result.Remove(result.Length - 1, 1)
        Return result.ToString()
    End Function

    Public Shared Function ParseDotNumbers(input As String) As Short()
        Dim result As New List(Of Short)
        Dim i, n As Short
        For Each c In input
            If c = "-"c Then
                result.Add(n Or &H2800S)
                n = 0
            Else
                i = Short.Parse(c)
                n += CShort(Math.Pow(2, i - 1))
            End If
        Next
        result.Add(n Or &H2800S)
        Return result.ToArray()
    End Function
End Class

Class NativeMethods
    <DllImport("liblouis.dll", BestFitMapping:=False, ThrowOnUnmappableChar:=True, CharSet:=CharSet.Unicode)>
    Public Shared Function lou_translateString(
        <MarshalAs(UnmanagedType.LPStr)> tableList As String,
        <MarshalAs(UnmanagedType.LPWStr)> inbuf As String,
         ByRef inlen As Integer,
        <MarshalAs(UnmanagedType.LPWStr)> outbuf As StringBuilder,
         ByRef outlen As Integer,
        typeform() As Typeforms,
        <MarshalAs(UnmanagedType.LPStr)> spacing As String,
         mode As Integer) As Integer
    End Function

    'int EXPORT_CALL lou_dotsToChar
    '(const char *tableList,
    ' widechar * inbuf,
    ' widechar * outbuf,
    ' int length,
    ' int mode)
    ' Requires proper input decoding
    <DllImport("liblouis.dll", BestFitMapping:=False, ThrowOnUnmappableChar:=True, CharSet:=CharSet.Unicode)>
    Public Shared Function lou_dotsToChar(
    <MarshalAs(UnmanagedType.LPStr)> tableList As String,
    inbuf() As Short,
    <MarshalAs(UnmanagedType.LPWStr), Out()> outbuf As StringBuilder,
    length As Int32,
    mode As Int32) As Int32
    End Function

    ' Uses unicode string as input
    <DllImport("liblouis.dll", BestFitMapping:=False, ThrowOnUnmappableChar:=True, CharSet:=CharSet.Unicode)>
    Public Shared Function lou_dotsToChar(
    <MarshalAs(UnmanagedType.LPStr)> tableList As String,
    <MarshalAs(UnmanagedType.LPWStr)> inbuf As String,
    <MarshalAs(UnmanagedType.LPWStr), Out()> outbuf As StringBuilder,
    length As Int32,
    mode As Int32) As Int32
    End Function

    'int EXPORT_CALL lou_charToDots
    '(const char *tableList,
    ' const widechar * inbuf,
    ' widechar * outbuf,
    ' int length,
    ' int mode)

    ' Allows result reinterpretation
    <DllImport("liblouis.dll", BestFitMapping:=False, ThrowOnUnmappableChar:=True, CharSet:=CharSet.Unicode)>
    Public Shared Function lou_charToDots(
    <MarshalAs(UnmanagedType.LPStr)> tableList As String,
    <MarshalAs(UnmanagedType.LPWStr)> inbuf As String,
    outbuf() As Short,
    length As Int32,
    mode As Int32) As Int32
    End Function

    ' Result will be a unicode string
    <DllImport("liblouis.dll", BestFitMapping:=False, ThrowOnUnmappableChar:=True, CharSet:=CharSet.Unicode)>
    Public Shared Function lou_charToDots(
    <MarshalAs(UnmanagedType.LPStr)> tableList As String,
    <MarshalAs(UnmanagedType.LPWStr)> inbuf As String,
    <MarshalAs(UnmanagedType.LPWStr), Out()> outbuf As StringBuilder,
    length As Int32,
    mode As Int32) As Int32
    End Function

    <DllImport("liblouis.dll", BestFitMapping:=False, ThrowOnUnmappableChar:=True, CharSet:=CharSet.Unicode)>
    Public Shared Function lou_hyphenate(
    <MarshalAs(UnmanagedType.LPStr)> tableList As String,
    <MarshalAs(UnmanagedType.LPWStr)> inbuf As String,
    inlen As Int32,
    <MarshalAs(UnmanagedType.LPWStr), Out()> hyphens As StringBuilder,
        mode As Int32) As Int32
    End Function

    Public Enum TranslationModes
        noContractions = 1
        compbrlAtCursor = 2
        dotsIO = 4
        comp8Dots = 8
        pass1Only = 16
        compbrlLeftCursor = 32
        otherTrans = 64
        ucBrl = 128
    End Enum

    Public Enum Typeforms As UShort
        plain_text = &H0
        italic = &H1     ' emph_1 
        underline = &H2  ' emph_2 
        bold = &H4       ' emph_3 
        emph_4 = &H8
        emph_5 = &H10
        emph_6 = &H20
        emph_7 = &H40
        emph_8 = &H80
        emph_9 = &H100
        emph_10 = &H200
        computer_braille = &H400
        no_translate = &H800
        no_contract = &H1000
    End Enum
End Class